/*
 * ArqWx::ListCtrl
 * v2.0 (2010/02)
 *
 * by TrV
 * contact: dev(at)arqendra(dot)net
 */


#ifndef ARQWX_LISTCTRL_H
#define ARQWX_LISTCTRL_H

#include <wx/listctrl.h>


namespace ArqWx
{

//** class ListCtrl
//
// wxWidgets' wxListCtrl class appended with additional useful methods
//
// derived from: wxListCtrl

class ListCtrl : public wxListCtrl
{
    public:
        // constructor(parent, ID, position, size, style, validator, name) : same attributes as wxListCtrl
        ListCtrl(
            wxWindow* parent = NULL,
            wxWindowID id = -1,
            const wxPoint& pos = wxDefaultPosition,
            const wxSize& size = wxDefaultSize,
            long style = wxLC_ICON,
            const wxValidator& validator = wxDefaultValidator,
            const wxString& name = wxListCtrlNameStr);

        virtual ~ListCtrl();

        long InsertItem(const wxListItem& info);
        long InsertItem(long index, const wxString& label);
        long InsertItem(long index, int imageIndex);
        long InsertItem(long index, const wxString& label, int imageIndex);

        bool DeleteItem(long item);

        bool SortItems(wxListCtrlCompare fnSortCallBack, long data);

        bool GetRowBackgroundColourAlternation() const;
        void SetRowBackgroundColourAlternation(bool rowBackgroundColoursAlternation);
        void SetRowBackgroundColours(wxColour evenColour, wxColour oddColour);

        long GetIdenticalItemsCount(const wxString& itemValue);

        long AddColumn(const wxListItem& info);
        long AddColumn(const wxString& heading, int format = wxLIST_FORMAT_LEFT, int width = -1);

        void ClearColumn(int colIdx);

        long GetSelectedItemIndex() const;

        wxString GetString(long itemIdx, int colIdx = 0) const;
        bool GetBoolean(long itemIdx, int colIdx = 0) const;
        long GetLong(long itemIdx, int colIdx = 0) const;
        double GetDouble(long itemIdx, int colIdx = 0) const;
        time_t GetDateTime(long itemIdx, int colIdx = 0) const;

        void GetItemRow(long itemIdx, wxString elements[]) const;
        void GetItemRow(long itemIdx, wxArrayString& values) const;

        void SetString(long itemIdx, int colIdx, const wxString& str);
        void SetBoolean(long itemIdx, int colIdx, bool bval);
        void SetLong(long itemIdx, int colIdx, long ival);
        void SetDouble(long itemIdx, int colIdx, double dval);
        void SetDateTime(long itemIdx, int colIdx, time_t tval);

        long AddItemRow(const wxString elements[]);
        long AddItemRow(const wxArrayString& elements);
        long SetItemRow(long itemIdx, const wxString elements[], bool autoInsertion = false);
        long SetItemRow(long itemIdx, const wxArrayString& elements, bool autoInsertion = false);

        void MoveItem(long itemIdx, long newIdx);

    private:
        void RepaintRowBackgroundColours(long startItemIdx = 0);
        void RepaintRowBackgroundDefaultColours();

        bool rowBackgroundColoursAlternation;
        wxColour* rowColours;
};

} // ArqWx namespace end

#endif // ARQWX_LISTCTRL_H
