/*
 * ArqWx::RegEx
 * v1.2 (2010/02)
 *
 * by TrV
 * contact: dev(at)arqendra(dot)net
 */


#ifndef ARQWX_REGEX_H
#define ARQWX_REGEX_H

#include <wx/wx.h>
#include <wx/regex.h>


namespace ArqWx
{

//** class RegEx
//
// Extends capacities of wxWidgets regular expressions management
// New functionalities:
// - extracts named groups
// - processes whole text with a single regex pattern, thus it can extracts as many matches are contained into whole text
//
// derived from: wxRegEx

class RegEx : public wxRegEx
{
    public:
        RegEx(const wxString& pattern, const wxString& text, int flags = wxRE_DEFAULT);
        RegEx(const wxString& pattern, int flags = wxRE_DEFAULT);

        void SetText(const wxString& text);

        wxString Groups(const wxString& text, const wxString& groupname);
        wxString Groups(const wxString& groupname);
        wxString Groups(const wxString& text, size_t index);
        wxString Groups(size_t index);

        size_t GetTotalMatchCount(const wxString& text) const;
        size_t GetTotalMatchCount() const;

        wxString GetTotalMatch(const wxString& text, size_t index = 0) const;
        wxString GetTotalMatch(size_t index = 0) const;

        bool Compile(const wxString& pattern, int flags = wxRE_DEFAULT);

        bool Matches(const wxChar* text, int flags = 0);
        bool Matches(const wxChar* text, int flags, size_t len);
        bool Matches(const wxString& text, int flags = 0);

    private:
        bool InitRegEx(const wxString& pattern, const wxString& text, int flags);
        void ExtractGroups();

        wxString pattern;
        wxString workingPattern;
        wxString text;
        int flags;
        int matchFlags;

        wxArrayString groupsList;
};

} // ArqWx namespace end

#endif // ARQWX_REGEX_H
