#include <wx/wx.h>
#include <wx/display.h>
#include <wx/textfile.h>
#include <wx/filedlg.h>
#include <wx/filename.h>
#include <wx/fileconf.h>
#include <wx/stdpaths.h>
#include <math.h>
#include <wx/aboutdlg.h>
#include "MyGUI.h"
#include "ArqWx.RegEx.h"

// turn debug mode on (1) to display a monitoring textctrl and enable debug mode
#define DEBUGMODE 0

int idMenuOpen = ::wxNewId();
int idMenuQuit = ::wxNewId();
int idMenuAnalyze = ::wxNewId();
int idMenuStopAnalysis = ::wxNewId();
int idMenuCreateCue = ::wxNewId();

int idMenuMoveUp = ::wxNewId();
int idMenuMoveDown = ::wxNewId();
int idMenuRemove = ::wxNewId();
int idMenuClearFilesList = ::wxNewId();
int idMenuCloseFile = ::wxNewId();

int idMenuRendererAuto = ::wxNewId();
int idMenuRendererDirectShow = ::wxNewId();
int idMenuRendererWMP = ::wxNewId();
int idMenuRendererMCI = ::wxNewId();
int idMenuRendererQuickTime = ::wxNewId();
int idMenuRendererGStreamer = ::wxNewId();

int idMenuTimeOut5 = ::wxNewId();
int idMenuTimeOut10 = ::wxNewId();
int idMenuTimeOut30 = ::wxNewId();
int idMenuTimeOut60 = ::wxNewId();
int idMenuTimeOut300 = ::wxNewId();

int idMenuAbout = ::wxNewId();

int idTextCtrlPlaylistFile = ::wxNewId();

int idBtnOpen = ::wxNewId();
int idBtnAnalyze = ::wxNewId();
int idBtnStop = ::wxNewId();
int idLstCtrlFiles = ::wxNewId();
int idChkBxCDText = ::wxNewId();
int idBtnCreateCue = ::wxNewId();

int idMediaCtrl = ::wxNewId();
int idTimeOutLoad = ::wxNewId();
int idTimeOutUnlock = ::wxNewId();

BEGIN_EVENT_TABLE(MyGUI, wxFrame)
    EVT_CLOSE(MyGUI::OnCloseWindow)

    EVT_DROP_FILES(MyGUI::OnDropFiles)

    EVT_MENU(idMenuOpen, MyGUI::OnMenuOpen)
    EVT_MENU(idMenuQuit, MyGUI::OnMenuQuit)

    EVT_MENU(idMenuAnalyze, MyGUI::OnMenuAnalyze)
    EVT_MENU(idMenuStopAnalysis, MyGUI::OnMenuStopAnalysis)
    EVT_MENU(idMenuCreateCue, MyGUI::OnMenuCreateCue)

    EVT_MENU(idMenuMoveUp, MyGUI::OnMenuMoveUp)
    EVT_MENU(idMenuMoveDown, MyGUI::OnMenuMoveDown)
    EVT_MENU(idMenuRemove, MyGUI::OnMenuRemove)
    EVT_MENU(idMenuClearFilesList, MyGUI::OnMenuClearFilesList)
    EVT_MENU(idMenuCloseFile, MyGUI::OnMenuCloseFile)

    EVT_MENU(idMenuRendererAuto, MyGUI::OnMenuRenderer)
    EVT_MENU(idMenuRendererDirectShow, MyGUI::OnMenuRenderer)
    EVT_MENU(idMenuRendererWMP, MyGUI::OnMenuRenderer)
    EVT_MENU(idMenuRendererMCI, MyGUI::OnMenuRenderer)
    EVT_MENU(idMenuRendererQuickTime, MyGUI::OnMenuRenderer)
    EVT_MENU(idMenuRendererGStreamer, MyGUI::OnMenuRenderer)

    EVT_MENU(idMenuTimeOut5, MyGUI::OnMenuTimeOut)
    EVT_MENU(idMenuTimeOut10, MyGUI::OnMenuTimeOut)
    EVT_MENU(idMenuTimeOut30, MyGUI::OnMenuTimeOut)
    EVT_MENU(idMenuTimeOut60, MyGUI::OnMenuTimeOut)
    EVT_MENU(idMenuTimeOut300, MyGUI::OnMenuTimeOut)

    EVT_MENU(idMenuAbout, MyGUI::OnMenuAbout)

    EVT_TEXT_ENTER(idTextCtrlPlaylistFile, MyGUI::OnKeyPressedOnPlaylist)
    EVT_BUTTON(idBtnOpen, MyGUI::OnClickOpen)

    EVT_BUTTON(idBtnAnalyze, MyGUI::OnClickAnalyze)
    EVT_BUTTON(idBtnStop, MyGUI::OnClickStopAnalysis)

    EVT_LIST_KEY_DOWN(idLstCtrlFiles, MyGUI::OnKeyPressedOnItem)

    EVT_CHECKBOX(idChkBxCDText, MyGUI::OnCheckCDText)
    EVT_BUTTON(idBtnCreateCue, MyGUI::OnClickCreateCue)

    EVT_MEDIA_LOADED(idMediaCtrl, MyGUI::OnMediaFileLoaded)
    EVT_TIMER(idTimeOutLoad, MyGUI::OnTimeOutLoad)
    EVT_TIMER(idTimeOutUnlock, MyGUI::OnTimeOutUnlock)
END_EVENT_TABLE()

//*** constructors & destructor

MyGUI::MyGUI(const wxString& applicationName, const wxPoint& pos, const wxSize& size)
: wxFrame((wxFrame*)NULL, -1, applicationName, wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE, applicationName)
, formatSeparator(",")
{
    this->LoadConfiguration();

    this->InitializeGUI();

    this->txtctrlPlaylistFile->SetValue(this->lastFile);
    this->rdbtnAnalysis[this->analysisMethod]->SetValue(true);
    this->rdbtnFilePath[this->filePathHandling]->SetValue(true);
    this->chkbxCDText->SetValue(this->cdText);
    this->txtctrlCDTextTemplate->SetValue(this->cdTextTemplate);

    // renderer initialization
    size_t rendererCount = 6;
    int rdrIndexes[] = {idMenuRendererAuto, idMenuRendererDirectShow, idMenuRendererWMP, idMenuRendererMCI, idMenuRendererQuickTime, idMenuRendererGStreamer};
    wxString rdrValues[] = {wxEmptyString, wxMEDIABACKEND_DIRECTSHOW, wxMEDIABACKEND_WMP10, wxMEDIABACKEND_MCI, wxMEDIABACKEND_QUICKTIME, wxMEDIABACKEND_GSTREAMER};
    for ( int i = 0 ; i < rendererCount ; i++ ) {
        this->rendererIndexes.Add(rdrIndexes[i]);
        this->rendererValues.Add(rdrValues[i]);
    }

    if ( (this->backend <= 0) || (this->backend > rendererCount - 1) ) { // if renderer is automatic or outside limits [0 ; 5]
        wxPlatformInfo pfInfo;
        if ( (pfInfo.GetOperatingSystemFamilyName() == "Windows") && pfInfo.CheckOSVersion(6, 0) ) { // for Windows OS version >= 6.0 (/ Vista)
            this->backend = 2;
        }
        else if ( pfInfo.GetOperatingSystemFamilyName() == "Macintosh" ) { // for Macintosh OS
            this->backend = 4;
        }
        else if ( pfInfo.GetOperatingSystemFamilyName() == "Unix" ) { // for Unix-based OS
            this->backend = 5;
        }
        else { // for other OS
            this->backend = 0; // sets automatic renderer
        }
    }

    this->menuOptions->Check(this->rendererIndexes[this->backend], true); // selects menu option

    // timeout initialization
    size_t toCount = 5;
    int toIndexes[] = {idMenuTimeOut5, idMenuTimeOut10, idMenuTimeOut30, idMenuTimeOut60, idMenuTimeOut300};
    int toValues[] = {5000, 10000, 30000, 60000, 300000};
    for ( int i = 0 ; i < toCount ; i++ ) {
        this->timeOutIndexes.Add(toIndexes[i]);
        this->timeOutValues.Add(toValues[i]);
    }

    if ( (this->loadTimeOut <= 0) || (this->loadTimeOut > toCount - 1) ) { // if time out value is outside limits [0 ; 4]
        this->loadTimeOut = 0;
    }

    this->menuOptions->Check(this->timeOutIndexes[this->loadTimeOut], true); // selects menu option

    this->UpdateGUI(0);
}

//*** GUI methods

//* void InitializeGUI()
//
// Initializes GUI
//
// parameters:
// - <none>
// returns:
// - <none>

void MyGUI::InitializeGUI()
{
    this->DragAcceptFiles(true);

    this->Move(this->pos);
    this->SetSize(this->size);

    this->SetIcon(wxICON(APPICON));

    // drawing of menu bar
    wxMenuBar* menuBar = new wxMenuBar();
    this->menuFile = new wxMenu();
    this->menuEdit = new wxMenu();
    this->menuOptions = new wxMenu();
    wxMenu* menuRenderer = new wxMenu();
    wxMenu* menuTimeOut = new wxMenu();
    wxMenu* menuHelp = new wxMenu();

    this->menuFile->Append(idMenuOpen, "Open", "Open a playlist or an audio file");
    this->menuFile->AppendSeparator();
    this->menuFile->Append(idMenuAnalyze, "Analyze", "Analyze files");
    this->menuFile->Append(idMenuStopAnalysis, "Stop", "Stop analysis");
    this->menuFile->AppendSeparator();
    this->menuFile->Append(idMenuCreateCue, "Create .cue", "Create .cue file");
    this->menuFile->AppendSeparator();
    this->menuFile->Append(idMenuQuit, "Quit", "Quit application");
    menuBar->Append(menuFile, "&File");

    this->menuEdit->Append(idMenuMoveUp, "Move up file", "Move up selected file in files list");
    this->menuEdit->Append(idMenuMoveDown, "Move down file", "Move down selected file in files list");
    this->menuEdit->Append(idMenuRemove, "Remove file", "Remove file from files list");
    this->menuEdit->AppendSeparator();
    this->menuEdit->Append(idMenuClearFilesList, "Clear files list", "Remove all items from files list");
    this->menuEdit->Append(idMenuCloseFile, "Close file", "Close file and reinitialize GUI");
    menuBar->Append(menuEdit, "&Edit");

    menuRenderer->AppendRadioItem(idMenuRendererAuto, "Automatic", "Set automatic renderer");
    menuRenderer->AppendRadioItem(idMenuRendererDirectShow, "ActiveMovie/DirectShow", "Set ActiveMovie/DirectShow as renderer (Windows only)");
    menuRenderer->AppendRadioItem(idMenuRendererWMP, "Windows Media Player 10", "Set Windows Media Player 10 as renderer (Windows only)");
    menuRenderer->AppendRadioItem(idMenuRendererMCI, "MCI", "Set Media Command Interface as renderer (Windows only)");
    menuRenderer->AppendRadioItem(idMenuRendererQuickTime, "QuickTime", "Set QuickTime as renderer (Mac only)");
    menuRenderer->AppendRadioItem(idMenuRendererGStreamer, "GStreamer", "Set GStreamer as renderer (Unix only)");
    menuOptions->AppendSubMenu(menuRenderer, "Renderer");

    menuTimeOut->AppendRadioItem(idMenuTimeOut5, "5", "5 seconds");
    menuTimeOut->AppendRadioItem(idMenuTimeOut10, "10", "10 seconds");
    menuTimeOut->AppendRadioItem(idMenuTimeOut30, "30", "30 seconds");
    menuTimeOut->AppendRadioItem(idMenuTimeOut60, "60", "60 seconds");
    menuTimeOut->AppendRadioItem(idMenuTimeOut300, "300", "300 seconds = 5 minutes");
    menuOptions->AppendSubMenu(menuTimeOut, "Time out");

    menuBar->Append(menuOptions, "&Options");

    menuHelp->Append(idMenuAbout, "About", "Information about application");
    menuBar->Append(menuHelp, "&Help");

    this->SetMenuBar(menuBar);

    this->CreateStatusBar();
    this->SetStatusText(wxEmptyString);

    // drawing of GUI
    this->panelMain = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL);
    wxBoxSizer* sizerMain = new wxBoxSizer(wxVERTICAL);
    this->panelMain->SetSizer(sizerMain);

        // playlist file
        this->panelPlaylistFile = new wxPanel(this->panelMain, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL);
        sizerMain->Add(this->panelPlaylistFile, 0, wxEXPAND);

        wxStaticBoxSizer* sizerPlaylistFile = new wxStaticBoxSizer(wxHORIZONTAL, this->panelPlaylistFile, "Playlist file");
        this->panelPlaylistFile->SetSizer(sizerPlaylistFile);

            this->txtctrlPlaylistFile = new wxTextCtrl(this->panelPlaylistFile, idTextCtrlPlaylistFile, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER);
            this->txtctrlPlaylistFile->SetToolTip("playlist file name");
            sizerPlaylistFile->Add(this->txtctrlPlaylistFile, 1);
            sizerPlaylistFile->AddSpacer(5);

            this->btnOpen = new wxButton(this->panelPlaylistFile, idBtnOpen, "Open", wxDefaultPosition, wxDefaultSize);
            this->btnOpen->SetToolTip("Open a playlist or an audio file");
            sizerPlaylistFile->Add(this->btnOpen, 0);

        sizerMain->AddSpacer(5);

        // analysis
        this->panelAnalysis = new wxPanel(this->panelMain, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL);
        sizerMain->Add(this->panelAnalysis, 0, wxEXPAND);

        wxStaticBoxSizer* sizerAnalysis = new wxStaticBoxSizer(wxHORIZONTAL, panelAnalysis, "Analysis");
        this->panelAnalysis->SetSizer(sizerAnalysis);

            this->panelSubAnalysis = new wxPanel(this->panelAnalysis, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL);
            sizerAnalysis->Add(this->panelSubAnalysis, 0);

            wxBoxSizer* sizerSubAnalysis = new wxBoxSizer(wxHORIZONTAL);
            this->panelSubAnalysis->SetSizer(sizerSubAnalysis);

                this->stxtAnalysisMethod = new wxStaticText(this->panelSubAnalysis, -1, "Analysis method:", wxDefaultPosition, wxDefaultSize);
                sizerSubAnalysis->Add(this->stxtAnalysisMethod, 0, wxALIGN_CENTER_VERTICAL);
                sizerSubAnalysis->AddSpacer(10);

                this->rdbtnAnalysis = new wxRadioButton*[2];

                this->rdbtnAnalysis[0] = new wxRadioButton(this->panelSubAnalysis, -1, "Independant", wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
                this->rdbtnAnalysis[0]->SetToolTip("Each file duration is computed independantly");
                sizerSubAnalysis->Add(this->rdbtnAnalysis[0], 0, wxALIGN_CENTER_VERTICAL);
                sizerSubAnalysis->AddSpacer(5);

                this->rdbtnAnalysis[1] = new wxRadioButton(this->panelSubAnalysis, -1, "Cumulative", wxDefaultPosition, wxDefaultSize);
                this->rdbtnAnalysis[1]->SetToolTip("Each file duration is added to previous one");
                sizerSubAnalysis->Add(this->rdbtnAnalysis[1], 0, wxALIGN_CENTER_VERTICAL);
                sizerSubAnalysis->AddSpacer(20);

                this->btnAnalyze = new wxButton(this->panelSubAnalysis, idBtnAnalyze, "Analyze", wxDefaultPosition, wxDefaultSize);
                this->btnAnalyze->SetToolTip("Analyze files");
                sizerSubAnalysis->Add(this->btnAnalyze, 0, wxALIGN_CENTER_VERTICAL);

            sizerAnalysis->AddSpacer(10);

            this->btnStop = new wxButton(this->panelAnalysis, idBtnStop, "Stop", wxDefaultPosition, wxDefaultSize);
            this->btnStop->SetToolTip("Stop analysis");
            sizerAnalysis->Add(this->btnStop, 0, wxALIGN_CENTER_VERTICAL);

        sizerMain->AddSpacer(10);

        // files list
        this->lstctrlFiles = new ArqWx::ListCtrl(this->panelMain, idLstCtrlFiles, wxDefaultPosition, wxDefaultSize, wxLC_REPORT + wxLC_SINGLE_SEL);
        this->lstctrlFiles->SetToolTip("<+>/<num+>: Move up file\n<->/<num->: Move down file\n<Del>/<BackSpace>: Remove file from list");
        sizerMain->Add(this->lstctrlFiles, 1, wxEXPAND);

        sizerMain->AddSpacer(5);

        // .cue creation
        this->panelCue = new wxPanel(this->panelMain, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL);
        sizerMain->Add(this->panelCue, 0, wxEXPAND);

        wxBoxSizer* sizerCue = new wxBoxSizer(wxHORIZONTAL);
        this->panelCue->SetSizer(sizerCue);

            wxPanel* panelCueParameters = new wxPanel(this->panelCue, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL);
            sizerCue->Add(panelCueParameters, 1, wxEXPAND);

            wxStaticBoxSizer* sizerCueParameters = new wxStaticBoxSizer(wxVERTICAL, panelCueParameters, ".cue file");
            panelCueParameters->SetSizer(sizerCueParameters);

                sizerCueParameters->AddSpacer(10);

                // file paths
                this->rdbtnFilePath = new wxRadioButton*[4];

                wxPanel* panelFilePath = new wxPanel(panelCueParameters, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL);
                sizerCueParameters->Add(panelFilePath, 0, wxEXPAND);

                wxBoxSizer* sizerFilePath = new wxBoxSizer(wxHORIZONTAL);
                panelFilePath->SetSizer(sizerFilePath);

                    this->stxtFilePath = new wxStaticText(panelFilePath, -1, "File path handling:", wxDefaultPosition, wxDefaultSize);
                    sizerFilePath->Add(this->stxtFilePath, 0, wxALIGN_CENTER_VERTICAL);
                    sizerFilePath->AddSpacer(10);

                    this->rdbtnFilePath[0] = new wxRadioButton(panelFilePath, -1, "Force relative", wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
                    this->rdbtnFilePath[0]->SetToolTip("Each file path is set to a relative path to .cue file");
                    sizerFilePath->Add(this->rdbtnFilePath[0], 0, wxALIGN_CENTER_VERTICAL);
                    sizerFilePath->AddSpacer(5);

                    this->rdbtnFilePath[1] = new wxRadioButton(panelFilePath, -1, "Force absolute", wxDefaultPosition, wxDefaultSize);
                    this->rdbtnFilePath[1]->SetToolTip("Each file path is set to an absolute path");
                    sizerFilePath->Add(this->rdbtnFilePath[1], 0, wxALIGN_CENTER_VERTICAL);
                    sizerFilePath->AddSpacer(5);

                    this->rdbtnFilePath[2] = new wxRadioButton(panelFilePath, -1, "Keep unchanged", wxDefaultPosition, wxDefaultSize);
                    this->rdbtnFilePath[2]->SetToolTip("Each file path is kept unchanged regarding path from files list");
                    sizerFilePath->Add(this->rdbtnFilePath[2], 0, wxALIGN_CENTER_VERTICAL);
                    sizerFilePath->AddSpacer(5);

                    this->rdbtnFilePath[3] = new wxRadioButton(panelFilePath, -1, "Remove", wxDefaultPosition, wxDefaultSize);
                    this->rdbtnFilePath[3]->SetToolTip("File paths are removed");
                    sizerFilePath->Add(this->rdbtnFilePath[3], 0, wxALIGN_CENTER_VERTICAL);

                sizerCueParameters->AddSpacer(15);

                // cd-text
                wxPanel* panelCDText = new wxPanel(panelCueParameters, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL);
                sizerCueParameters->Add(panelCDText, 0, wxEXPAND);

                wxBoxSizer* sizerCDText = new wxBoxSizer(wxHORIZONTAL);
                panelCDText->SetSizer(sizerCDText);

                    this->chkbxCDText = new wxCheckBox(panelCDText, idChkBxCDText, "CD-Text", wxDefaultPosition, wxDefaultSize);
                    this->chkbxCDText->SetToolTip("Add CD-Text information");
                    sizerCDText->Add(this->chkbxCDText, 0, wxALIGN_CENTER_VERTICAL);
                    sizerCDText->AddSpacer(10);

                    this->txtctrlCDTextTemplate = new wxTextCtrl(panelCDText, -1, wxEmptyString, wxDefaultPosition, wxDefaultSize);
                    this->txtctrlCDTextTemplate->SetToolTip("Filename template:\n- *1: track title (smallest possible)\n- ?1: track title (longest possible)\n- *2: track performer (smallest possible)\n- ?2: track performer (longest possible)\n- *0: unused (smallest possible)\n- ?0: unused (longest possible)");
                    sizerCDText->Add(this->txtctrlCDTextTemplate, 1);
                    sizerCDText->AddSpacer(10);

                    this->txtctrlAlbumTitle = new wxTextCtrl(panelCDText, -1, "Album title", wxDefaultPosition, wxDefaultSize);
                    this->txtctrlAlbumTitle->SetToolTip("Album title");
                    sizerCDText->Add(this->txtctrlAlbumTitle, 2);
                    sizerCDText->AddSpacer(5);

                    this->txtctrlAlbumPerformer = new wxTextCtrl(panelCDText, -1, "Album performer", wxDefaultPosition, wxDefaultSize);
                    this->txtctrlAlbumPerformer->SetToolTip("Album performer");
                    sizerCDText->Add(this->txtctrlAlbumPerformer, 2);

                sizerCueParameters->AddSpacer(5);

            sizerCue->AddSpacer(5);

            this->btnCreateCue = new wxButton(this->panelCue, idBtnCreateCue, "\nCreate .cue\n\n", wxDefaultPosition, wxDefaultSize);
            this->btnCreateCue->SetToolTip("Create .cue file");
            sizerCue->Add(this->btnCreateCue, 0, wxALIGN_CENTRE_VERTICAL);

            sizerCue->AddSpacer(5);

        sizerMain->AddSpacer(5);

        // debug window
        #if DEBUGMODE == 1
        this->txtctrlDebug = new wxTextCtrl(this->panelMain, -1, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE | wxTE_READONLY);
        sizerMain->Add(this->txtctrlDebug, 1, wxEXPAND, 0);
        #endif

    // initializations
    int colCount = 6;
    wxListItem itemCol; // listitem element to initialize headers and columns
    wxString colHeaders[] = {"Index", "Filename", "Path", "Length", "Start", "End"};
    long colWidths[] = {50, 50, 50, 50, 50, 50};
    ArqWx::RegEx colWidthsRegEx("([^" + this->formatSeparator + "]+)");
    long width;

    this->lstctrlFiles->SetRowBackgroundColourAlternation(true);

    colWidthsRegEx.SetText(this->columnWidths);

    // initialization of column width values
    for ( int i = 1 ; (i <= colWidthsRegEx.GetTotalMatchCount()) && (i <= colCount) ; i++ ) { // for each match
        colWidthsRegEx.GetTotalMatch(i).ToLong(&width);
        colWidths[i - 1] = width;
    }

    // initializations of columns
    for ( int i = 0 ; i < colCount ; i++ ) {
        itemCol.SetText(colHeaders[i]); // sets header
        itemCol.SetWidth(colWidths[i]); // sets width
        this->lstctrlFiles->InsertColumn(i, itemCol);
    }
}

//* void UpdateGUI(int step)
//
// Updates GUI: enables and/or disables some graphic objects
//
// parameters:
// - step (int): conversion step number
// returns:
// - <none>

void MyGUI::UpdateGUI(int step)
{
    switch ( step ) {
        case 0: // initial state
            this->menuFile->Enable(idMenuAnalyze, false);
            this->menuFile->Enable(idMenuStopAnalysis, false);
            this->menuFile->Enable(idMenuCreateCue, false);

            this->panelAnalysis->Enable(false);
            this->panelCue->Enable(false);
            break;

        case 1: // playlist file has been loaded
            this->menuFile->Enable(idMenuOpen, true);
            this->menuFile->Enable(idMenuAnalyze, true);
            this->menuFile->Enable(idMenuStopAnalysis, false);

            this->menuEdit->Enable(idMenuMoveUp, true);
            this->menuEdit->Enable(idMenuMoveDown, true);
            this->menuEdit->Enable(idMenuRemove, true);
            this->menuEdit->Enable(idMenuClearFilesList, true);
            this->menuEdit->Enable(idMenuCloseFile, true);

            this->menuOptions->Enable(idMenuRendererAuto, true);
            this->menuOptions->Enable(idMenuRendererDirectShow, true);
            this->menuOptions->Enable(idMenuRendererWMP, true);
            this->menuOptions->Enable(idMenuRendererMCI, true);
            this->menuOptions->Enable(idMenuRendererQuickTime, true);
            this->menuOptions->Enable(idMenuRendererGStreamer, true);

            this->menuOptions->Enable(idMenuTimeOut5, true);
            this->menuOptions->Enable(idMenuTimeOut10, true);
            this->menuOptions->Enable(idMenuTimeOut30, true);
            this->menuOptions->Enable(idMenuTimeOut60, true);
            this->menuOptions->Enable(idMenuTimeOut300, true);

            this->panelPlaylistFile->Enable(true);
            this->panelAnalysis->Enable(true);
            this->panelSubAnalysis->Enable(true);
            this->btnStop->Enable(false);

            this->panelCue->Enable(false);
            break;

        case 2: // analysis has been done
            this->menuFile->Enable(idMenuCreateCue, true);
            this->menuEdit->Enable(idMenuCloseFile, true);

            this->panelCue->Enable(true);
            if ( !this->chkbxCDText->GetValue() ) {
                this->txtctrlCDTextTemplate->Enable(false);
                this->txtctrlAlbumTitle->Enable(false);
                this->txtctrlAlbumPerformer->Enable(false);
            }
    }
}

void MyGUI::QuitApplication()
{
    this->SaveConfiguration(); // saves configuration

    // application exit
    this->Destroy(); // destroys frame, thus application
}

//*** application management methods

//* void LoadConfiguration()
//
// Retrieves parameters' values from ini file
//
// parameters:
// - <none>
// returns:
// - <none>

void MyGUI::LoadConfiguration()
{
    int posX, posY; // default position
    int sizeW, sizeH; // default size
    wxFileConfig* configFile; // configuration file

    ::wxSetWorkingDirectory( wxFileName(wxStandardPaths::Get().GetExecutablePath()).GetPath() ); // sets working directory as current application directory

    // read of saved values
    configFile = new wxFileConfig("", "", this->GetName() + "_config.ini", "", wxCONFIG_USE_RELATIVE_PATH); // initializes configuration file

    if ( configFile->GetNumberOfGroups() == 0 && configFile->GetNumberOfEntries() == 0 ) { // if configuration file structure is not OK (/configuration file does not exist or is corrupted)
        configFile->DeleteAll();
        configFile->Write("/PositionSize/X", 50); // saves x position
        configFile->Write("/PositionSize/Y", 50); // saves y position
        configFile->Write("/PositionSize/W", 630); // saves x size
        configFile->Write("/PositionSize/H", 440); // saves y size
        configFile->Write("/PositionSize/ColumnWidths", "50,160,180,70,70,70"); // saves column widths
        configFile->Write("/Options/PlaylistFileFormats", "m3u,m3u8,txt"); // saves playlist file formats value
        configFile->Write("/Options/AudioFileFormats", "wav,mp3,wma,aac,ogg,flac,wv,ape"); // saves audio file formats value
        configFile->Write("/Options/LastFile", ""); // saves last playlist file value
        configFile->Write("/Options/AnalysisMethod", 0); // saves analysis method value
        configFile->Write("/Options/FilePathHandling",0); // saves file path handling value
        configFile->Write("/Options/CDText", false); // saves cd-text value
        configFile->Write("/Options/CDTextTemplate", ""); // saves cd-text template value
        configFile->Write("/Options/Renderer", "0"); // saves renderer value
        configFile->Write("/Options/TimeOut", "0"); // saves timeout value
        configFile->Flush(true); // writes changes to file
    }

    configFile->Read("/PositionSize/X", &posX); // retrieves x position
    configFile->Read("/PositionSize/Y", &posY); // retrieves y position
    this->pos = wxPoint(posX, posY); // sets position

    configFile->Read("/PositionSize/W", &sizeW); // retrieves width size
    configFile->Read("/PositionSize/H", &sizeH); // retrieves height size
    this->size = wxSize(sizeW, sizeH); // sets size

    configFile->Read("/PositionSize/ColumnWidths", &columnWidths); // retrieves columns width

    configFile->Read("/Options/PlaylistFileFormats", &this->playlistFileFormats); // retrieves playlist file formats value
    configFile->Read("/Options/AudioFileFormats", &this->audioFileFormats); // retrieves audio file formats value

    configFile->Read("/Options/LastFile", &this->lastFile); // retrieves last playlist file value
    configFile->Read("/Options/AnalysisMethod", &this->analysisMethod); // retrieves analysis method value
    configFile->Read("/Options/FilePathHandling", &this->filePathHandling); // retrieves file path handling value
    configFile->Read("/Options/CDText", &this->cdText); // retrieves cd-text value
    configFile->Read("/Options/CDTextTemplate", &this->cdTextTemplate); // retrieves cd-text template value
    configFile->Read("/Options/Renderer", &this->backend); // retrieves renderer value
    configFile->Read("/Options/TimeOut", &this->loadTimeOut); // retrieves load timeout value

    delete configFile;

    // formatting of playlist file formats list
    this->playlistFileFormats.Replace(" ", "");

    if ( !this->playlistFileFormats.StartsWith(this->formatSeparator) ) {
        this->playlistFileFormats = this->formatSeparator + this->playlistFileFormats;
    }
    if ( !this->playlistFileFormats.EndsWith(this->formatSeparator) ) {
        this->playlistFileFormats = this->playlistFileFormats + this->formatSeparator;
    }

    // formatting of audio file formats list
    this->audioFileFormats.Replace(" ", "");

    if ( !this->audioFileFormats.StartsWith(this->formatSeparator) ) {
        this->audioFileFormats = this->formatSeparator + this->audioFileFormats;
    }
    if ( !this->audioFileFormats.EndsWith(this->formatSeparator) ) {
        this->audioFileFormats = this->audioFileFormats + this->formatSeparator;
    }
}

//* void SaveConfiguration()
//
// Saves parameters' values into ini file
//
// parameters:
// - <none>
// returns:
// - <none>

void MyGUI::SaveConfiguration()
{
    int posX, posY;
    int sizeW, sizeH;
    int margin = 50;
    wxString renderer;
    wxFileConfig* configFile;

    ::wxSetWorkingDirectory( wxFileName(wxStandardPaths::Get().GetExecutablePath()).GetPath() ); // sets working directory as current application directory
    configFile = new wxFileConfig("", "", this->GetName() + "_config.ini", "", wxCONFIG_USE_RELATIVE_PATH); // initializes configuration file

    this->GetPosition(&posX, &posY); // gets position
    this->GetSize(&sizeW, &sizeH); // gets size

    if ( (posX > 0 - sizeW + margin) &&
         (posX < wxDisplay().GetGeometry().GetWidth() - margin) &&
         (posY > 0 - sizeH + margin) &&
         (posY < wxDisplay().GetGeometry().GetHeight() - margin) )
    { // if position is inside limits

        configFile->Write("/PositionSize/X", posX); // saves x position
        configFile->Write("/PositionSize/Y", posY); // saves y position
        configFile->Write("/PositionSize/W", sizeW); // saves width size
        configFile->Write("/PositionSize/H", sizeH); // saves height size

        this->columnWidths.Clear();
        for ( int i = 0 ; i < this->lstctrlFiles->GetColumnCount() ; i++ ) {
            this->columnWidths += wxString::Format("%d", this->lstctrlFiles->GetColumnWidth(i)) + " ";
        }
        this->columnWidths.Trim(false).Trim(true);
        this->columnWidths.Replace(" ", ",");
        configFile->Write("/PositionSize/ColumnWidths", this->columnWidths); // saves columns width
    }

    configFile->Write("/Options/LastFile", this->txtctrlPlaylistFile->GetValue()); // saves last playlist file value

    // retrievation of analysis method
    for ( int i = 1 ; i >= 0 ; i-- ) {
        if ( this->rdbtnAnalysis[i]->GetValue() ) {
            this->analysisMethod = i;
        }
    }
    configFile->Write("/Options/AnalysisMethod", this->analysisMethod); // saves analysis method value

    // retrievation of file path handling
    for ( int i = 3 ; i >= 0 ; i-- ) {
        if ( this->rdbtnFilePath[i]->GetValue() ) {
            this->filePathHandling = i;
        }
    }
    configFile->Write("/Options/FilePathHandling", this->filePathHandling); // saves file path handling value

    configFile->Write("/Options/CDText", this->chkbxCDText->GetValue()); // saves cd-text value
    configFile->Write("/Options/CDTextTemplate", this->txtctrlCDTextTemplate->GetValue()); // saves cd-text template value

    configFile->Write("/Options/Renderer", this->backend); // saves renderer value
    configFile->Write("/Options/TimeOut", this->loadTimeOut); // saves load timeout value

    configFile->Flush(); // writes changes to file

    delete configFile;
}

//*** specific methods

//* void OpenFile()
//
// Opens files
//
// parameters:
// - <none>
// returns:
// - <none>

void MyGUI::OpenFile()
{
    ArqWx::RegEx extensionRegEx("([^" + this->formatSeparator + "]+)");
    wxString extension, playlistExtensions, playlistExtensionsList, audioExtensions, audioExtensionsList;
    wxString wildCard;
    wxFileDialog* filedlgOpenFiles; // dialog to choose files to open

    // creation of wildcard
    extensionRegEx.SetText(this->playlistFileFormats);

    for ( int i = 1 ; i <= extensionRegEx.GetTotalMatchCount() ; i++ ) {
        extension = extensionRegEx.GetTotalMatch(i);
        playlistExtensionsList += "*." + extension.Lower() + " "; // final separators ('|') are noted as blank space (' ')
        playlistExtensions += extension.Upper() + "_files *." + extension.Lower() + " "; // final separators ('|') are noted as blank space (' '), final blank spaces (' ') are noted as underscore ('_')
    }
    playlistExtensionsList.Trim(true).Trim(false); // removes leading and trailing "separators" (= blank spaces)
    playlistExtensionsList.Replace(" ", ";"); // replaces temporary separators (= blank spaces) with final separators (';')

    playlistExtensions.Trim(true).Trim(false); // removes leading and trailing "separators" (= blank spaces)
    playlistExtensions.Replace(" ", "|"); // replaces temporary separators (blank spaces) with final separators ('|')
    playlistExtensions.Replace("_", " "); // replaces temporary blank spaces (= underscores) with real blank spaces

    extensionRegEx.SetText(this->audioFileFormats);

    for ( int i = 1 ; i <= extensionRegEx.GetTotalMatchCount() ; i++ ) {
        extension = extensionRegEx.GetTotalMatch(i);
        audioExtensionsList += "*." + extension.Lower() + " "; // final separators ('|') are noted as blank space (' ')
        audioExtensions += extension.Upper() + "_files *." + extension.Lower() + " "; // final separators ('|') are noted as blank space (' '), final blank spaces (' ') are noted as underscore ('_')
    }
    audioExtensionsList.Trim(true).Trim(false); // removes leading and trailing "separators" (= blank spaces)
    audioExtensionsList.Replace(" ", ";"); // replaces temporary separators (= blank spaces) with final separators (';')

    audioExtensions.Trim(true).Trim(false); // removes leading and trailing "separators" (= blank spaces)
    audioExtensions.Replace(" ", "|"); // replaces temporary separators (blank spaces) with final separators ('|')
    audioExtensions.Replace("_", " "); // replaces temporary blank spaces (= underscores) with real blank spaces

    wildCard = "All supported formats|" + playlistExtensionsList + ";" + audioExtensionsList + "|Supported playlist files|" + playlistExtensionsList + "|" + playlistExtensions + "|Supported audio files|" + audioExtensionsList + "|" + audioExtensions + "|All extensions|*.*"; // builds wildcard string

    // dialog opening
    filedlgOpenFiles = new wxFileDialog(this, "Open a playlist or an audio file", wxFileName(this->txtctrlPlaylistFile->GetValue()).GetPath(), this->txtctrlPlaylistFile->GetValue(), wildCard, wxFD_OPEN + wxFD_FILE_MUST_EXIST + wxFD_MULTIPLE); // opens the dialog
    if ( filedlgOpenFiles->ShowModal() == wxID_OK ) { // if a file has been chosen
        wxArrayString selectedFiles;

        filedlgOpenFiles->GetPaths(selectedFiles); // retrieves file path & name and displays it

        for ( int i = 0 ; i < selectedFiles.GetCount() ; i++ ) { // for each selected file
            this->LoadFile(selectedFiles[i]); // loads file
        }
    }

    delete filedlgOpenFiles;
}

//* void LoadFile()
//
// Loads file
//
// parameters:
// - fileName (wxString): name of file to load
// returns:
// - (bool): file loaded flag

void MyGUI::LoadFile(wxString fileName)
{
    ArqWx::RegEx formatRegEx("(.+)\\.(.+)");
    wxString fileFormat;
    bool supportedFormat = false;

    if ( formatRegEx.Matches(fileName) ) { // if current dropped file has an extension
        fileFormat = formatRegEx.GetMatch(fileName, 2); // retrieves extension

        ArqWx::RegEx extensionsRegEx("(" + this->formatSeparator + fileFormat + this->formatSeparator + ")", wxRE_EXTENDED + wxRE_ICASE);

        if ( extensionsRegEx.Matches(this->playlistFileFormats) ) { // if extension is an attended playlist format
            this->LoadPlaylistFile(fileName);

            supportedFormat = true;
        }
        else if ( extensionsRegEx.Matches(this->audioFileFormats) ) { // if extension is an attended audio format
            this->LoadAudioFile(fileName);

            supportedFormat = true;
        }
        else {
            this->SetStatusText("\"" + fileName + "\": unknown or unsupported file format");
        }
    }
}

//* void LoadPlaylistFile(wxString fileName)
//
// Loads playlist file
//
// parameters:
// - fileName (wxString): file path of playlist file to load
// returns:
// - <none>

void MyGUI::LoadPlaylistFile(wxString fileName)
{
    wxFileName playlistFileName(fileName);
    wxTextFile* playlistFile; // file to read playlist file content

    if ( playlistFileName.IsRelative() ) {
        playlistFileName.MakeAbsolute();
    }

    playlistFile = new wxTextFile(playlistFileName.GetFullPath()); // creates a text file object upon given filename

    if ( playlistFile->Exists() ) { // if chosen file exists
        this->txtctrlPlaylistFile->SetValue(playlistFileName.GetFullPath());

        if ( playlistFile->Open() ) { // if chosen file can be opened
            wxString line; // one line from playlist file
            size_t nLines = 0;

            for ( line = playlistFile->GetFirstLine() ; !playlistFile->Eof() ; line = playlistFile->GetNextLine() ) { // for each line of the chosen file
                wxFileName filePath(line); // creates filename upon entry

                if ( filePath.IsOk() ) { // if line is a file path
                    this->LoadAudioFile(line);
                    nLines++; // counts entries
                }
                #if DEBUGMODE == 1
                this->txtctrlDebug->AppendText(line + "\n");
                #endif
            }

            this->UpdateGUI(1);

            this->SetStatusText("File reading done: " + wxString::Format("%d", nLines) + " entries found"); // displays a message

            #if DEBUGMODE == 1
            this->txtctrlDebug->AppendText(wxString::Format("%d", nLines) + " lines\n");
            #endif

            playlistFile->Close();
        }
        else { // if file can not be opened
            this->SetStatusText("File can not be opened");
        }
    }
    else { // if file doesn't exist
        this->SetStatusText("File does not exist");
    }

    delete playlistFile;
}

//* void LoadAudioFile(wxString fileName)
//
// Loads audio file
//
// parameters:
// - fileName (wxString): file path of audio file to load
// returns:
// - <none>

void MyGUI::LoadAudioFile(wxString fileName)
{
    ArqWx::RegEx formatRegEx("(.+)\\.(.+)");
    wxString fileFormat;

    if ( formatRegEx.Matches(fileName) ) { // if current dropped file has an extension
        fileFormat = formatRegEx.GetMatch(fileName, 2); // retrieves extension

        ArqWx::RegEx extensionsRegEx("(" + this->formatSeparator + fileFormat + this->formatSeparator + ")", wxRE_EXTENDED + wxRE_ICASE);

        if ( extensionsRegEx.Matches(this->audioFileFormats) ) { // if extension is an attended audio format
            wxFileName playlistFilePathName(this->txtctrlPlaylistFile->GetValue());
            playlistFilePathName.MakeAbsolute();

            wxFileName filePathName(fileName); // creates filename upon entry
            filePathName.MakeAbsolute(playlistFilePathName.GetPathWithSep());

            this->lstctrlFiles->InsertItem(this->lstctrlFiles->GetItemCount(), wxString::Format("%d", this->lstctrlFiles->GetItemCount() + 1)); // inserts index
            this->lstctrlFiles->SetItem(this->lstctrlFiles->GetItemCount() - 1, 1, filePathName.GetFullName()); // inserts filename

            this->lstctrlFiles->SetItem(this->lstctrlFiles->GetItemCount() - 1, 2, filePathName.GetPathWithSep()); // inserts path

            this->UpdateGUI(1);
        }
    }
}

//* void MoveUpFile()
//
// Moves up selected file in files list
//
// parameters:
// - <none>
// returns:
// - <none>

void MyGUI::MoveUpFile()
{
    long currentFileIdx; // selected file index

    currentFileIdx = this->lstctrlFiles->GetSelectedItemIndex(); // gets selected file index
    if ( currentFileIdx != -1 ) { // if a file is selected
        if ( currentFileIdx >= 1 ) { // if selected file is not the first one in the list
            this->lstctrlFiles->MoveItem(currentFileIdx, currentFileIdx - 1); // moves file item

            this->lstctrlFiles->SetLong(currentFileIdx - 1, 0, currentFileIdx); // re-indexes moved item
            this->lstctrlFiles->SetLong(currentFileIdx, 0, currentFileIdx + 1); // re-indexes other moved item

            this->lstctrlFiles->SetItemState(currentFileIdx - 1, wxLIST_STATE_SELECTED + wxLIST_STATE_FOCUSED, wxLIST_STATE_SELECTED + wxLIST_STATE_FOCUSED); // selects moved file item
            this->lstctrlFiles->EnsureVisible(currentFileIdx - 1); // ensures moved item is visible in listcontrol
        }
    }
    else { // if no file is selected
        this->SetStatusText("No file is selected");
    }
}

//* void MoveDownFile()
//
// Moves down selected file in files list
//
// parameters:
// - <none>
// returns:
// - <none>

void MyGUI::MoveDownFile()
{
    long currentFileIdx; // selected file index

    currentFileIdx = this->lstctrlFiles->GetSelectedItemIndex(); // gets selected file index
    if ( currentFileIdx != -1 ) { // if a file is selected
        if ( currentFileIdx <= this->lstctrlFiles->GetItemCount() - 2 ) { // if selected file is not the last one in the list
            this->lstctrlFiles->MoveItem(currentFileIdx, currentFileIdx + 1); // moves file item

            this->lstctrlFiles->SetLong(currentFileIdx + 1, 0, currentFileIdx + 2); // re-indexes moved item
            this->lstctrlFiles->SetLong(currentFileIdx, 0, currentFileIdx + 1); // re-indexes other moved item
            this->lstctrlFiles->SetItemState(currentFileIdx + 1, wxLIST_STATE_SELECTED + wxLIST_STATE_FOCUSED, wxLIST_STATE_SELECTED + wxLIST_STATE_FOCUSED); // selects moved file item
            this->lstctrlFiles->EnsureVisible(currentFileIdx + 1); // ensures moved item is visible in listcontrol
        }
    }
    else { // if no file is selected
        this->SetStatusText("No file is selected");
    }
}

//* void RemoveFile()
//
// Removes selected file from files list
//
// parameters:
// - <none>
// returns:
// - <none>

void MyGUI::RemoveFile()
{
    long currentFileIdx; // selected file index

    currentFileIdx = this->lstctrlFiles->GetSelectedItemIndex(); // gets selected file index
    if ( currentFileIdx != -1 ) { // if a file is selected
        this->lstctrlFiles->DeleteItem(currentFileIdx); // deletes file item

        for ( int i = currentFileIdx ; i < this->lstctrlFiles->GetItemCount() ; i++ ) { // for each item after removed one
            this->lstctrlFiles->SetLong(i, 0, i + 1); // re-indexes it
        }

        if ( currentFileIdx == this->lstctrlFiles->GetItemCount() ) { // if selected file item was last one
            currentFileIdx--; // chooses previous file item
        }

        if ( currentFileIdx >= 0 ) { // if chosen file item exists
            this->lstctrlFiles->SetItemState(currentFileIdx, wxLIST_STATE_SELECTED + wxLIST_STATE_FOCUSED, wxLIST_STATE_SELECTED + wxLIST_STATE_FOCUSED); // selects file item
            this->lstctrlFiles->EnsureVisible(currentFileIdx); // ensures item is visible in listcontrol
        }
        else {
            this->UpdateGUI(1);
        }
    }
    else { // if no file is selected
        this->SetStatusText("No file is selected");
    }
}

//* void InitializeAnalysis()
//
// Initializes analysis
//
// parameters:
// - <none>
// returns:
// - <none>

void MyGUI::InitializeAnalysis()
{
    // GUI update
    this->UpdateGUI(0);
    this->lstctrlFiles->ClearColumn(3);
    this->lstctrlFiles->ClearColumn(4);
    this->lstctrlFiles->ClearColumn(5);

    this->menuFile->Enable(idMenuOpen, false);
    this->menuFile->Enable(idMenuStopAnalysis, true);

    this->menuEdit->Enable(idMenuMoveUp, false);
    this->menuEdit->Enable(idMenuMoveDown, false);
    this->menuEdit->Enable(idMenuRemove, false);
    this->menuEdit->Enable(idMenuClearFilesList, false);
    this->menuEdit->Enable(idMenuCloseFile, false);

    this->menuOptions->Enable(idMenuRendererAuto, false);
    this->menuOptions->Enable(idMenuRendererDirectShow, false);
    this->menuOptions->Enable(idMenuRendererWMP, false);
    this->menuOptions->Enable(idMenuRendererMCI, false);
    this->menuOptions->Enable(idMenuRendererQuickTime, false);
    this->menuOptions->Enable(idMenuRendererGStreamer, false);

    this->menuOptions->Enable(idMenuTimeOut5, false);
    this->menuOptions->Enable(idMenuTimeOut10, false);
    this->menuOptions->Enable(idMenuTimeOut30, false);
    this->menuOptions->Enable(idMenuTimeOut60, false);
    this->menuOptions->Enable(idMenuTimeOut300, false);

    this->panelPlaylistFile->Enable(false);
    this->panelAnalysis->Enable(true);
    this->panelSubAnalysis->Enable(false);
    this->btnStop->Enable(true);

    this->SetStatusText("Starting file analysis. Please wait...");

    // initializations
    this->currentFile = 0;
    this->cumulativeStart = 0;
    this->cumulativeEnd = 0;

    this->analysisOK = true;
    this->analysisError = false;

    // launch of analysis
    if ( this->unlockTimer != NULL ) { // if timer object has previously been created
        if ( this->unlockTimer->IsRunning() ) { // if unlock timer is still running
            this->unlockTimer->Stop(); // stops it
        }
        delete this->unlockTimer; // deletes unlock timer object
    }

    if ( this->mctrl != NULL ) { // if previous media control has not been destroyed
        this->mctrl->Destroy();
        this->mctrl = NULL;
    }

    this->mctrl = new wxMediaCtrl(this->panelMain, idMediaCtrl, wxEmptyString, wxPoint(0,0), wxSize(0,0), 0, this->rendererValues[this->backend]);
    this->Analyze(); // launches analysis
}

//* void Analyze()
//
// Analyzes current file' timing information
//
// parameters:
// - <none>
// returns:
// - <none>

void MyGUI::Analyze()
{
    // files are processed one after one using onMediaLoaded event:
    // - first analysis launches first media file loading
    // - when media file has ended up loading, an event is fired off (see MyGUI::OnMediaLoaded())
    // - the event launches next media file loading (see MyGUI::OnMediaLoaded())
    // - and so on until the end of media files list
    if ( this->analysisOK && (this->currentFile < this->lstctrlFiles->GetItemCount()) ) { // if current file is into the list
        this->SetStatusText("Processing file #" + wxString::Format("%d/%d", this->currentFile + 1, this->lstctrlFiles->GetItemCount()) + " '" + this->lstctrlFiles->GetString(this->currentFile, 1) + "'...");

        this->lstctrlFiles->EnsureVisible(this->currentFile); // ensures current item is visible in listcontrol

        // file length computation start
        wxFileName filePathName(this->lstctrlFiles->GetString(this->currentFile, 2), this->lstctrlFiles->GetString(this->currentFile, 1));

        #if DEBUGMODE == 1
        this->txtctrlDebug->AppendText(filePathName.GetFullPath());
        #endif

        if ( wxFile::Exists(filePathName.GetFullPath()) ) { // if file exists
            if ( filePathName.IsRelative() ) {
                filePathName.MakeAbsolute();
            }

            #if DEBUGMODE == 1
            this->txtctrlDebug->AppendText(": file exists (" + filePathName.GetFullPath() + ")\r\n");
            #endif

            // timeout watcher start
            if ( this->loadTimer != NULL ) { // if timer object has previously been created
                if ( this->loadTimer->IsRunning() ) { // if timer is still running
                    this->loadTimer->Stop(); // stops it
                }
                delete this->loadTimer; // deletes timer object
            }
            this->loadTimer = new wxTimer(this, idTimeOutLoad); // creates load timer
            this->loadTimer->Start(this->timeOutValues[this->loadTimeOut], wxTIMER_ONE_SHOT); // starts load timer

            // launches computation
            this->mctrl->Load(filePathName.GetFullPath());
        }
        else { // if file does not exist
            #if DEBUGMODE == 1
            this->txtctrlDebug->AppendText(": file does not exist\r\n");
            #endif

            this->analysisError = true; // sets error flag as true
            this->lstctrlFiles->SetItem(this->currentFile, 2, "NO FILE"); // displays error information

            // next file management
            this->currentFile++; // sets next file as current file
            this->Analyze(); // launches analysis for next file
        }
    }
    else { // if current file is not into the list (= last file has been processed)
        this->FinalizeAnalysis();
    }
}

//* void FinalizeAnalysis()
//
// Finalizes analysis
//
// parameters:
// - <none>
// returns:
// - <none>

void MyGUI::FinalizeAnalysis()
{
    // timeout watcher cancel
    if ( this->loadTimer != NULL ) { // if timer object has previously been created
        if ( this->loadTimer->IsRunning() ) { // if timer is still running
            this->loadTimer->Stop(); // stops it
        }
        delete this->loadTimer; // deletes timer object
        this->loadTimer = NULL; // ensures timer object is reinitialized
    }

    // GUI update
    this->UpdateGUI(1);

    if ( this->analysisOK ) { // if analysis has not been stopped
        if ( !this->analysisError ) { // if no error occurred during analysis
            wxString analysisMethod;

            // GUI update
            this->UpdateGUI(2);

            // final message display
            if ( this->rdbtnAnalysis[0]->GetValue() ) {
                analysisMethod = this->rdbtnAnalysis[0]->GetLabelText();
            }
            else {
                analysisMethod = this->rdbtnAnalysis[1]->GetLabelText();
            }

            this->SetStatusText(analysisMethod + " analysis done"); // displays a message
        }
        else { // if an error occurred during file length computation
            this->SetStatusText("Error(s) during analysis");
        }
    }
    else {
        this->SetStatusText("Analysis has been aborted");
    }

    // mediacontrol management
    this->mctrl->Stop();

    // on some OS, destroying mediacontrol object here seems to create memory leak leading to application crash
    // so destruction of mediacontrol is done in another method, launched by a timer
    this->unlockTimer = new wxTimer(this, idTimeOutUnlock); // creates unlock timer
    this->unlockTimer->Start(0, wxTIMER_ONE_SHOT); // starts unlock timer
}

//* void StopAnalysis()
//
// Stops analysis
//
// parameters:
// - <none>
// returns:
// - <none>

void MyGUI::StopAnalysis()
{
    this->menuFile->Enable(idMenuStopAnalysis, false);
    this->btnStop->Enable(false);

    this->analysisOK = false;
}

//* wxString GetMSmDuration(long milliSecondsDuration)
//
// Converts a duration in milliseconds (mS) to a duration in MM:SS:mmmm (M: minutes, S: seconds, m: milliseconds)
// (currently unused)
//
// parameters:
// - milliSecondsDuration (long): duration in milliconds
// returns:
// - (wxString): duration in MM:SS:mmmm (M: minutes, S: seconds, m: milliseconds) as string

wxString MyGUI::GetMSmDuration(long milliSecondsDuration)
{
    wxString MSmSDuration;

    long seconds = milliSecondsDuration / 1000; // computes seconds (part 1)
    long milliSeconds = milliSecondsDuration - seconds * 1000; // computes milliseconds
    long minutes = seconds / 60; // computes minutes
    seconds = seconds - minutes * 60; // computes seconds (part 2)

    MSmSDuration.Printf("%02d:%02d.%03d", minutes, seconds, milliSeconds); // constructs string

    return MSmSDuration; // returns string
}


//* wxString GetMSFDuration(long milliSecondsDuration)
//
// Converts a duration in milliseconds (mS) to a duration in MM:SS:FF (M: minutes, S: seconds, F: frames (1 frame = 1/75 seconds))
//
// parameters:
// - milliSecondsDuration (long): duration in milliconds
// returns:
// - (wxString): duration in MM:SS:FF as string

wxString MyGUI::GetMSFDuration(long milliSecondsDuration)
{
    wxString MSFDuration;

    long seconds = milliSecondsDuration / 1000; // computes seconds (part 1)
    long milliSeconds = milliSecondsDuration - seconds * 1000; // computes milliseconds
    double frames1 = (double) milliSeconds * 75 / 1000; // computes frames (part 1)
    long frames2 = (long) frames1; // computes frames (part 2)
    if ( frames2 + 0.5 < frames1 ) {
        frames2++; // computes frames with rounding (part 3)

        if ( frames2 == 75 ) {
            frames2 = 0;
            seconds++;
        }
    }
    long minutes = seconds / 60; // computes minutes
    seconds = seconds - minutes * 60; // computes seconds (part 2)

    MSFDuration.Printf("%02d:%02d:%02d", minutes, seconds, frames2); // constructs string

    return MSFDuration; // returns string
}

//* void CreateCue()
//
// Creates .cue file
//
// parameters:
// - <none>
// returns:
// - <none>

void MyGUI::CreateCue()
{
    bool saveFileOverwrite = false; // writing flag
    wxFileName playlistFileName(this->txtctrlPlaylistFile->GetValue()); // retrieves playlist filename
    wxFileName cueFileName;
    wxTextFile* cueTxtFile = new wxTextFile(); // destination file

    if ( this->txtctrlPlaylistFile->GetValue().IsEmpty() ) {
        cueFileName.Assign(this->lstctrlFiles->GetString(0, 1));
        cueFileName.Assign(this->lstctrlFiles->GetString(0, 2), cueFileName.GetName(), "cue"); // constructs .cue filename using 1st file

        //wxFileName::SetCwd(this->lstctrlFiles->GetString(0, 2)); // sets current directory
    }
    else {
        cueFileName.Assign(playlistFileName.GetPathWithSep() + playlistFileName.GetName() + ".cue"); // constructs .cue filename using playlist file

        //wxFileName::SetCwd(playlistFileName.GetPathWithSep()); // sets current directory
    }

    #if DEBUGMODE == 1
    this->txtctrlDebug->AppendText(::wxGetCwd() + "\r\n");
    #endif

    wxFileDialog* filedlgCueFile; // dialog to choose .cue file to save

    filedlgCueFile = new wxFileDialog(this, "Save .cue file", cueFileName.GetPath(), cueFileName.GetFullPath(), "CUE files|*.cue|All files|*.*", wxFD_SAVE + wxFD_OVERWRITE_PROMPT); // opens the dialog
    if ( filedlgCueFile->ShowModal() == wxID_OK ) { // if a file has been chosen
        bool cueNameOk = false;
        wxString cueName = filedlgCueFile->GetPath(); // retrieves file path & name and displays it

        cueTxtFile = new wxTextFile(cueName);

        if ( cueTxtFile->Exists() ) { // if destination file exists
            cueNameOk = cueTxtFile->Open(cueName); //  tries to open
        }
        else { // if destination file does not exist
            cueNameOk = cueTxtFile->Create(cueName); // tries to create file
        }

        if ( cueNameOk ) { // if .cue file has either been opened or been created
            cueTxtFile->Clear(); // empties file content

            // .cue creation
            wxString tab = "  ";

            // tags pre-processing
            wxString cdTextTemplate; // user cd-text template
            wxString tagsTextTmp[] = { "unused", "title", "performer" }; // text tags list
            wxArrayString tagsText(3, tagsTextTmp); // text tags list

            wxString albumTitle = "Album title";
            wxString albumPerformer = "Album performer";

            cdTextTemplate = this->txtctrlCDTextTemplate->GetValue(); // initializes cd-text template

            cdTextTemplate.Replace("(", "\\(");
            cdTextTemplate.Replace(")", "\\)");
            cdTextTemplate.Replace("[", "\\[");
            cdTextTemplate.Replace("]", "\\]");
            cdTextTemplate.Replace(".", "\\.");

            for ( int i = 0 ; i < tagsText.GetCount() ; i++ ) { // for each tag
                cdTextTemplate.Replace(wxString::Format("*%d", i), "(?<" + tagsText[i] + ">.+?)"); // replaces '*x' (with 0 <= x <= 2) with named group (smallest possible)
                cdTextTemplate.Replace(wxString::Format("?%d", i), "(?<" + tagsText[i] + ">.+)"); // replaces '?x' (with 0 <= x <= 2) with named group (longest possible)
            }

            #if DEBUGMODE == 1
            this->txtctrlDebug->AppendText("modified cd-text template: " + cdTextTemplate + "\r\n");
            #endif

            // writing
            if ( this->chkbxCDText->GetValue() ) { // if cd-text is selected
                if ( !this->txtctrlAlbumTitle->GetValue().IsEmpty() ) {
                    albumTitle = this->txtctrlAlbumTitle->GetValue();
                }
                if ( !this->txtctrlAlbumPerformer->GetValue().IsEmpty() ) {
                    albumPerformer = this->txtctrlAlbumPerformer->GetValue();
                }

                cueTxtFile->AddLine("TITLE \"" + albumTitle + "\""); // writes album title information
                cueTxtFile->AddLine("PERFORMER \"" + albumPerformer + "\""); // writes album performer information
            }

            for ( int item = 0 ; item < this->lstctrlFiles->GetItemCount() ; item++ ) { // for each file from the list
                wxFileName filePathName(this->lstctrlFiles->GetString(item, 2), this->lstctrlFiles->GetString(item, 1)); // constructs a filename with path & name from the list
                wxString track; // track number

                // path processing
                #if DEBUGMODE == 1
                this->txtctrlDebug->AppendText(wxString::Format("%d: ", item) + filePathName.GetFullPath() + "\r\n");
                #endif

                if ( this->rdbtnFilePath[0]->GetValue() ) { // if files path method is 'force relative'
                    if ( filePathName.IsAbsolute() ) { // if current filename is absolute
                        filePathName.MakeRelativeTo(cueFileName.GetPath()); // changes it to relative
                    }
                }
                else if ( this->rdbtnFilePath[1]->GetValue() ) { // if files path method is 'force absolute'
                    if ( filePathName.IsRelative() ) { // if current filename is relative
                        filePathName.MakeAbsolute(); // changes it to absolute
                    }
                }
                else if ( this->rdbtnFilePath[3]->GetValue() ) { // if files path method is 'remove'
                    filePathName.Assign("", filePathName.GetFullName()); // removes path
                }

                // track number processing
                track.Printf("%02d", item + 1); // initializes track number

                // tags processing
                #if DEBUGMODE == 1
                this->txtctrlDebug->AppendText(wxString::Format("%d: ", item) + filePathName.GetFullName() + "\r\n");
                #endif

                ArqWx::RegEx tagsRegEx(cdTextTemplate, filePathName.GetFullName(), wxRE_ADVANCED);

                // cd-text information writing
                cueTxtFile->AddLine("FILE \"" + filePathName.GetFullPath() + "\" WAVE"); // writes file information
                cueTxtFile->AddLine(tab + "TRACK " + track + " AUDIO"); // writes track information

                if ( this->chkbxCDText->GetValue() ) { // if cd-text is enabled
                    wxString trackTitle = albumTitle + " " + track;
                    wxString trackPerformer = albumPerformer;

                    if ( !tagsRegEx.Groups("title").IsEmpty() ) {
                        trackTitle = tagsRegEx.Groups("title");
                    }

                    if ( !tagsRegEx.Groups("performer").IsEmpty() ) {
                        trackPerformer = tagsRegEx.Groups("performer");
                    }

                    cueTxtFile->AddLine(tab + tab + "TITLE \"" + trackTitle + "\""); // writes title information
                    cueTxtFile->AddLine(tab + tab + "PERFORMER \"" + trackPerformer + "\""); // writes performer information
                }

                cueTxtFile->AddLine(tab + tab + "INDEX 01 " + this->lstctrlFiles->GetString(item, 4)); // writes index information
            }

            cueTxtFile->Write(); // writes changes to .cue file
            cueTxtFile->Close(); // closes .cue file

            this->SetStatusText(".cue file saved as " + cueName); // displays a message

            delete cueTxtFile;
        }
        else { // if destination file can't be opened
            this->SetStatusText("File can not be created"); // sends alert
        }
    }

    //::wxSetWorkingDirectory( wxFileName(wxStandardPaths::Get().GetExecutablePath()).GetPath() ); // sets working directory as current application directory

    delete filedlgCueFile;
}

//*** frame events

void MyGUI::OnCloseWindow(wxCloseEvent& WXUNUSED(event))
{
    this->QuitApplication(); // quits application
}

void MyGUI::OnDropFiles(wxDropFilesEvent& event)
{
    for ( int i = 0 ; i < event.GetNumberOfFiles() ; i++ ) { // for each dropped file
        this->LoadFile(event.GetFiles()[i]);
    }
}

//*** menu events

void MyGUI::OnMenuOpen(wxCommandEvent& WXUNUSED(event))
{
    this->OpenFile();
}

void MyGUI::OnMenuQuit(wxCommandEvent& WXUNUSED(event))
{
    this->QuitApplication();
}

void MyGUI::OnMenuAnalyze(wxCommandEvent& WXUNUSED(event))
{
    this->InitializeAnalysis();
}

void MyGUI::OnMenuStopAnalysis(wxCommandEvent& WXUNUSED(event))
{
    this->StopAnalysis();
}

void MyGUI::OnMenuCreateCue(wxCommandEvent& WXUNUSED(event))
{
    this->CreateCue();
}

void MyGUI::OnMenuMoveUp(wxCommandEvent& WXUNUSED(event))
{
    this->MoveUpFile();
}

void MyGUI::OnMenuMoveDown(wxCommandEvent& WXUNUSED(event))
{
    this->MoveDownFile();
}

void MyGUI::OnMenuRemove(wxCommandEvent& WXUNUSED(event))
{
    this->RemoveFile();
}

void MyGUI::OnMenuClearFilesList(wxCommandEvent& WXUNUSED(event))
{
    this->lstctrlFiles->DeleteAllItems();
    this->UpdateGUI(0);
}

void MyGUI::OnMenuCloseFile(wxCommandEvent& WXUNUSED(event))
{
    this->lstctrlFiles->DeleteAllItems();
    this->txtctrlPlaylistFile->SetValue("");
    this->UpdateGUI(0);
}

void MyGUI::OnMenuRenderer(wxCommandEvent& event)
{
    int idx = this->rendererIndexes.Index(event.GetId());
    if ( idx != wxNOT_FOUND ) {
        this->backend = idx;
    }
}

void MyGUI::OnMenuTimeOut(wxCommandEvent& event)
{
    int idx = this->timeOutIndexes.Index(event.GetId());
    if ( idx != wxNOT_FOUND ) {
        this->loadTimeOut = idx;
    }
}

void MyGUI::OnMenuAbout(wxCommandEvent& WXUNUSED(event))
{
    wxAboutDialogInfo aboutDlgInfo;
    aboutDlgInfo.SetName("m3u2cue");
    aboutDlgInfo.SetVersion("v1.8");
    aboutDlgInfo.SetCopyright("(2011/07)");
    aboutDlgInfo.SetDescription("\n.m3u to .cue converter\n&& .cue creator\n\nby TrV\ncontact: dev(at)arqendra(dot)net\t\n\nDeveloped with:\n- Code::Blocks 10.05\n- MinGW 4.4.1\n- wxWidgets 2.8.12\n- NConvert 4.95\n- UPX 3.07w\n");
    aboutDlgInfo.SetWebSite("http://dev.arqendra.net");

    ::wxAboutBox(aboutDlgInfo);
}

//*** button events

void MyGUI::OnKeyPressedOnPlaylist(wxCommandEvent& WXUNUSED(event))
{
    this->LoadFile(this->txtctrlPlaylistFile->GetValue());
}

void MyGUI::OnClickOpen(wxCommandEvent& WXUNUSED(event))
{
    this->OpenFile();
}

void MyGUI::OnClickAnalyze(wxCommandEvent& WXUNUSED(event))
{
    this->InitializeAnalysis();
}

void MyGUI::OnClickStopAnalysis(wxCommandEvent& WXUNUSED(event))
{
    this->StopAnalysis();
}

void MyGUI::OnKeyPressedOnItem(wxListEvent& event)
{
    switch ( event.GetKeyCode() ) {
        case 8: // del
        case 127: // suppr
            this->RemoveFile();
            break;
        case 43: // plus
        case 388: // numpad plus
            this->MoveDownFile();
            break;
        case 54: // minus
        case 390: // numpad minus
            this->MoveUpFile();
            break;
        default:
            break;
    }
}

void MyGUI::OnCheckCDText(wxCommandEvent& event)
{
    if ( event.IsChecked() ) {
        this->txtctrlCDTextTemplate->Enable(true); // enables 'CD-Text template' textctrl
        this->txtctrlAlbumTitle->Enable(true); // enables 'Album title' textctrl
        this->txtctrlAlbumPerformer->Enable(true); // enables 'Album performer' textctrl
    }
    else {
        this->txtctrlCDTextTemplate->Enable(false); // disables 'CD-Text template' textctrl
        this->txtctrlAlbumTitle->Enable(false); // disables 'Album title' textctrl
        this->txtctrlAlbumPerformer->Enable(false); // disables 'Album performer' textctrl
    }
}

void MyGUI::OnClickCreateCue(wxCommandEvent& WXUNUSED(event))
{
    this->CreateCue();
}

//*** other events

void MyGUI::OnTimeOutLoad(wxTimerEvent& WXUNUSED(event))
{
    // error management
    this->analysisError = true; // sets error flag as true
    this->lstctrlFiles->SetItem(this->currentFile, 3, "ERROR"); // displays error information

    // next file management
    this->currentFile++; // sets next file as current file
    this->Analyze(); // launches analysis for next file
}

void MyGUI::OnTimeOutUnlock(wxTimerEvent& WXUNUSED(event))
{
    if ( this->mctrl != NULL ) {
        this->mctrl->Destroy();
        this->mctrl = NULL;
    }
}

// to use a media file, one has to wait until native OS' filters has completely loaded it
// so duration (and other manipulations of the file like play) can only be retrieved once the file has been loaded

void MyGUI::OnMediaFileLoaded(wxMediaEvent& WXUNUSED(event))
{
    // media file length management
    long fileDuration = this->mctrl->Length(); // retrieves its duration

    #if DEBUGMODE == 1
    this->txtctrlDebug->AppendText(wxString::Format("%ld", fileDuration) + "\n");
    #endif

    if ( this->rdbtnAnalysis[0]->GetValue() ) { // if analysis method is 'independant'
        // re-initializes time values for each loaded file
        this->cumulativeStart = 0;
        this->cumulativeEnd = 0;
    }
    this->cumulativeEnd += fileDuration; // sets the end of the file (cumulative or absolute)
    this->lstctrlFiles->SetItem(this->currentFile, 3, this->GetMSFDuration(fileDuration)); // displays MM:SS:FF-converted length
    this->lstctrlFiles->SetItem(this->currentFile, 4, this->GetMSFDuration(this->cumulativeStart)); // displays MM:SS:FF-converted start
    this->lstctrlFiles->SetItem(this->currentFile, 5, this->GetMSFDuration(this->cumulativeEnd)); // displays MM:SS:FF-converted end

    // next file management
    this->cumulativeStart += fileDuration; // updates start for next file

    this->currentFile++; // sets next file as current file
    this->Analyze(); // launches analysis for next file
}
