#ifndef MYGUI_H
#define MYGUI_H

#include <wx/wx.h>
#include <wx/dynarray.h>
#include <wx/mediactrl.h>
#include "ArqWx.ListCtrl.h"

//** class MyGUI
//
// Main frame
//
// derived from: wxFrame

class MyGUI: public wxFrame
{
    public:
        // constructor(title, position, size)
        MyGUI(const wxString& title, const wxPoint& pos, const wxSize& size);

        void QuitApplication(); // quits application

        void OnMenuOpen(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Open' menu item is selected
        void OnMenuQuit(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Quit' menu item is selected

        void OnMenuAnalyze(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Analyze' menu item is selected
        void OnMenuStopAnalysis(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Stop analysis' menu item is selected
        void OnMenuCreateCue(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Create .cue' menu item is selected

        void OnMenuMoveUp(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Move up file' menu item is selected
        void OnMenuMoveDown(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Move down file' menu item is selected
        void OnMenuRemove(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Remove file' menu item is selected
        void OnMenuClearFilesList(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Clear files list' menu item is selected
        void OnMenuCloseFile(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Close file' menu item is selected

        void OnMenuRenderer(wxCommandEvent& event); // actions executed when 'Renderer' item is changed
        void OnMenuTimeOut(wxCommandEvent& event); // actions executed when 'TimeOut' item is changed

        void OnMenuAbout(wxCommandEvent& WXUNUSED(event)); // actions executed when 'About' menu item is selected

        void OnCloseWindow(wxCloseEvent& WXUNUSED(event)); // actions executed when the frame is closed
        void OnDropFiles(wxDropFilesEvent& WXUNUSED(event)); // actions executed when a file is drag'n'dropped from files explorer

        void OnKeyPressedOnPlaylist(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Enter' is pressed
        void OnClickOpen(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Open' button is clicked

        void OnClickAnalyze(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Analyze' button is clicked
        void OnClickStopAnalysis(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Stop analysis' button is clicked

        void OnKeyPressedOnItem(wxListEvent& event);

        void OnMediaFileLoaded(wxMediaEvent& WXUNUSED(event)); // actions executed when media file is loaded
        void OnTimeOutLoad(wxTimerEvent& WXUNUSED(event)); // actions executed when load time out has been reached
        void OnTimeOutUnlock(wxTimerEvent& WXUNUSED(event)); // actions executed when unlock time out has been reached

        void OnCheckCDText(wxCommandEvent& WXUNUSED(event)); // actions executed when 'CD-Text' checkbox is checked
        void OnClickCreateCue(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Create .cue' button is clicked

    private:
        void LoadConfiguration(); // loads parameters' values from configuration file
        void SaveConfiguration(); // saves parameters' values into configuration file

        void InitializeGUI();
        void UpdateGUI(int step);

        void OpenFile(); // browses to choose a playlist file
        void LoadFile(wxString fileName);
        void LoadPlaylistFile(wxString fileName);
        void LoadAudioFile(wxString fileName);

        void MoveUpFile();
        void MoveDownFile();
        void RemoveFile();

        void InitializeAnalysis(); // inits analysis
        void Analyze(); // launches analysis for one file
        void FinalizeAnalysis(); // finalyzes analysis
        void StopAnalysis(); // stops analysis

        void CreateCue(); // creates .cue file

        wxString GetMSmDuration(long milliSecondsDuration); // returns duration as MM:SS:mmmm (M: minutes, S: seconds, m: milliseconds)
        wxString GetMSFDuration(long milliSecondsDuration); // returns duration as MM:SS:FF (M: minutes, S: seconds, F: frames (1 frame = 1/75 seconds))

        wxPoint pos; // frame position
        wxSize size; // frame size

        wxPanel* panelMain; // main panel

        wxMenu* menuFile; // 'File' menu
        wxMenu* menuEdit; // 'Edit' menu
        wxMenu* menuOptions; // 'Options' menu

        wxPanel* panelPlaylistFile; // 'playlist file' panel

        wxTextCtrl* txtctrlPlaylistFile; // chosen playlist file
        wxButton* btnOpen; // 'Open' button

        wxPanel* panelAnalysis; // 'Analysis' panel
        wxPanel* panelSubAnalysis;

        wxStaticText* stxtAnalysisMethod; // analysis method selection
        wxRadioButton** rdbtnAnalysis; // analysis radiobuttons
        wxButton* btnAnalyze; // 'Analyze' button
        wxButton* btnStop; // 'Stop' button

        ArqWx::ListCtrl* lstctrlFiles; // information for files from playlist file (filename - path - length - start - end)

        wxPanel* panelCue; // '.cue file' panel

        wxStaticText* stxtFilePath; // 'Files path' text
        wxRadioButton** rdbtnFilePath; // file path radiobuttons

        wxCheckBox* chkbxCDText; // 'CD-Text' checkbox
        wxTextCtrl* txtctrlCDTextTemplate; // template textctrl
        wxTextCtrl* txtctrlAlbumTitle; // album title textctrl
        wxTextCtrl* txtctrlAlbumPerformer; //album performer textctrl

        wxButton* btnCreateCue; // 'Create .cue' button

        wxTextCtrl* txtctrlDebug; // to display information during development (see MyGUI.cpp > #define DEBUGMODE)

        wxMediaCtrl* mctrl; // mediactrl object to analyze files
        int backend; // mediactrl backend (media renderer)

        wxArrayInt rendererIndexes;
        wxArrayString rendererValues;

        wxArrayInt timeOutIndexes;
        wxArrayInt timeOutValues;

        const wxString formatSeparator;
        wxString playlistFileFormats;
        wxString audioFileFormats;

        wxString columnWidths;

        wxString lastFile;
        int analysisMethod;
        int filePathHandling;
        bool cdText;
        wxString cdTextTemplate;

        bool analysisOK; // analysis flag
        bool analysisError; // analysis error

        int loadTimeOut; // load file timeout
        wxTimer* loadTimer; // load file timer

        wxTimer* unlockTimer; // mediacontrol unlock delay timer

        wxFileName m3uFileName;

        size_t currentFile; // current analyzed file
        size_t cumulativeStart; // cumulative start of current file
        size_t cumulativeEnd; // cumulative end of current file

    DECLARE_EVENT_TABLE()
};

#endif // MYGUI_H
